/***************************************************************************
 *   Copyright (C) 2007-2008 by Peter Semiletov                            *
 *   peter.semiletov@gmail.com                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
** Copyright (C) 2006-2008 Trolltech ASA. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License versions 2.0 or 3.0 as published by the Free Software
** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file.
*/
/*
code from qPEditor:

*   Copyright (C) 2007 by Michael Protasov
*   mik-protasov@anyqsoft.com

*/
/*
Diego Iastrubni <elcuco@kde.org> //some GPL'ed code from new-editor-diego-3, found on qtcentre forum

*/


#include <QtGui>
#include <QDomDocument>

#include "document.h"
#include "utils.h"



void CDocument::update_status ()
{
  if (! cursor_xy_visible)
     {
      holder->l_status_bar->setText (charset);
      return;
     }

  QTextBlock block = textEdit->document()->begin();
  int cursorOffset = textEdit->textCursor().position();
  int off = 0;
  int x = 1, y = 1;

  while ( cursorOffset>=(off + block.length()) )
       {
        off += block.length();
        block = block.next();
        y+= 1;
       }

  x = cursorOffset - off +1;
  holder->l_status_bar->setText (QString::number (y) + ":" + QString::number (x) + " " + charset);
}


void CDocument::update_title (bool fullname)
{
  QMainWindow *w = qobject_cast<QMainWindow *>(holder->parent_wnd);

  if (fullname)
     w->setWindowTitle (file_name);
  else
     w->setWindowTitle (strippedName(file_name));
}


void CDocument::reload (const QString &enc)
{
  QFile file (file_name);

  if (! file.open(QFile::ReadOnly | QFile::Text))
     {
      holder->log->log (tr("cannot open %1 because of: %2")
                           .arg(file_name)
                           .arg(file.errorString()));
      return;
     }

  QTextStream in(&file);
  charset = enc;
  in.setCodec(charset.toUtf8());

  textEdit->setPlainText(in.readAll());

  set_hl ();

  textEdit->document()->setModified (false);

  holder->log->log (tr("%1 is reloaded") .arg(file_name));
}


bool CDocument::open_file (const QString &fileName, const QString &codec)
{
  QFile file (fileName);

  if (! file.open(QFile::ReadOnly | QFile::Text))
     {
      holder->log->log (tr("cannot open %1 because of: %2")
                           .arg(fileName)
                           .arg(file.errorString()));
      return false;
     }

  QTextStream in(&file);

  if (fileName.contains (holder->dir_config))
    {
     charset = "UTF-8";
     in.setCodec ("UTF-8");
    }
  else
      {
       charset = codec.toUtf8();
       in.setCodec (charset.toUtf8());
      }

  textEdit->setPlainText(in.readAll());

  file_name = fileName;

  set_tab_caption (strippedName (file_name));
  set_hl ();
  set_markup_mode ();
  textEdit->document()->setModified (false);

  holder->log->log (tr("%1 is opened") .arg(file_name), "smile01.mng");
//  holder->watcher.addPath (file_name);

  int size = holder->recent_files.size ();
  int found = -1;
  for (int i = 0; i < size; i++)
     {
      if (holder->recent_files[i].contains (file_name))
         {
          found = i;
          break;
         }
     }

  if (found != -1)
     holder->recent_files.removeAt (found);

  return true;
}


bool CDocument::save_with_name (const QString &fileName, const QString &codec)
{
  QFile file (fileName);
  if (! file.open (QFile::WriteOnly | QFile::Text))
     {
      holder->log->log (tr("cannot save %1 because of: %2")
                           .arg(fileName)
                           .arg(file.errorString()));

      return false;
     }

  QTextStream out (&file);

  if (fileName.contains (holder->dir_config))
    {
     charset = "UTF-8";
     out.setCodec ("UTF-8");
    }
  else
      {
       charset = codec.toUtf8();
       out.setCodec (charset.toUtf8());
      }

  out << textEdit->toPlainText();

  file_name = fileName;

  set_tab_caption (strippedName (file_name));

  holder->log->log (tr("%1 is saved") .arg(file_name), "pgaz01.mng");

//  holder->watcher.addPath (file_name);

  update_title ();
  update_status ();

  textEdit->document()->setModified (false);

  return true;
}


CDocument::CDocument(QObject *parent): QObject (parent)
{
  QTime t = QTime::currentTime();
  QString fname = tr ("new") + "[" + t.toString("hh-mm-ss") + "]";

  markup_mode = "HTML";

  file_name = fname;

  cursor_xy_visible = true;
  charset = "UTF-8";
  text_to_search = "";
  position = 0;
}


CDocument::~CDocument()
{
  if (file_name == holder->fname_crapbook && textEdit->document()->isModified())
     save_with_name_plain (file_name);
  else
  if (textEdit->document()->isModified() && file_exists (file_name))
     {
      int ret = QMessageBox::warning(0, "TEA",
                        tr("%1 has been modified.\n"
                        "Do you want to save your changes?")
                        .arg(file_name),
                        QMessageBox::Ok | QMessageBox::Default,
                        QMessageBox::Cancel | QMessageBox::Escape);

      if (ret == QMessageBox::Ok)
         save_with_name (file_name, charset);

     }


  if (file_name != holder->fname_crapbook)
     {
      holder->add_to_recent (this);
      holder->update_recent_menu ();
     }


  int i = holder->tab_widget->indexOf (tab_page);
  holder->tab_widget->removeTab (i);

//CTEAEdit *we = static_cast<CTEAEdit*>(holder->tab_widget->widget(i));
//we->close();

//  holder->watcher.removePath (file_name);
//  delete textEdit;
textEdit->deleteLater();
}


void CDocument::create_new (void)
{
  textEdit = new CTEAEdit;

  textEdit->doc = this;
  textEdit->setAcceptRichText (false);
  highlighter = NULL;

  int tab_index = holder->tab_widget->addTab (textEdit, file_name);
  tab_page = holder->tab_widget->widget (tab_index);
  textEdit->setFocus(Qt::MouseFocusReason);

  //completer = new QCompleter(this);

  //completer->setModel(new QStringListModel(qstring_load (":/text-data/wordlist.txt").trimmed().split ("\n"), completer));
 // QString s = qstring_load (holder->fname_compdict);

  //completer->setModel(new QStringListModel(qstring_load (holder->fname_compdict).trimmed().split ("\n"), completer));

 // completer->setModelSorting(QCompleter::CaseInsensitivelySortedModel);
  //completer->setCaseSensitivity(Qt::CaseInsensitive);
  //completer->setWrapAround(false);
  //textEdit->setCompleter(completer);
}


int CDocument::get_tab_idx (void)
{
  return holder->tab_widget->indexOf (tab_page);
}


void CDocument::set_tab_caption (const QString &fileName)
{
  holder->tab_widget->setTabText (get_tab_idx (), fileName);
}


document_holder::document_holder()
{
  qDebug() << "document_holder";
//  connect(&watcher, SIGNAL(fileChanged(const QString &)), this, SLOT(watcher_fileChanged (const QString &)));
}


void document_holder::reload_recent_list (void)
{
  if (!file_exists (recent_list_fname))
     return;

  recent_files = qstring_load (recent_list_fname).split ("\n");
}


document_holder::~document_holder()
{
  qDebug() << "~document_holder";

  while (! list.isEmpty())
          delete list.takeFirst();

 qstring_save (recent_list_fname, recent_files.join("\n"));
}


CDocument* document_holder::create_new (void)
{
  CDocument *doc = new CDocument;

  doc->holder = this;
  doc->markup_mode = markup_mode;
  list.append (doc);

  doc->create_new ();


  tab_widget->setCurrentIndex (tab_widget->indexOf (doc->tab_page));
  apply_settings_single (doc);

  doc->update_title ();
  doc->update_status ();
  return doc;
}


CDocument* document_holder::open_file (const QString &fileName, const QString &codec)
{
  CDocument *d;
  int c = list.size();

  for (int i = 0; i < c; i++)
      {
       d = list[i];

       if (d->file_name == fileName)
          {
           tab_widget->setCurrentIndex (tab_widget->indexOf (d->tab_page));
           return d;
          }
      }

  QFile file (fileName);
  if (! file.exists())
     return NULL;

  CDocument *doc = create_new ();
  doc->open_file (fileName, codec);
  doc->update_status();
  doc->update_title();

  tab_widget->setCurrentIndex (tab_widget->indexOf (doc->tab_page));
  apply_settings_single (doc);
  return doc;
}


void document_holder::close_by_idx (int i)
{
  if (i < 0)
     return;

  CDocument *d = list[i];
  list.removeAt (i);
  delete d;
}


void document_holder::watcher_fileChanged (const QString & path)
{
  //qDebug() << path;
//   qDebug() << watcher.files() << watcher.directories();
/*
  int ret = QMessageBox::warning(NULL, "TEA",
                                 tr("%1 was modified by other program\n Do you want to reload?")
                                    .arg(path),
                                    QMessageBox::Yes | QMessageBox::Default,
                                    QMessageBox::Cancel | QMessageBox::Escape);

  if (ret == QMessageBox::Cancel)
      return;

  CDocument *d;
  int c = list.size();

  for (int i = 0; i < c; i++)
      {
       d = list[i];

       if (d->file_name == path)
          {
           tab_widget->setCurrentIndex (tab_widget->indexOf (d->tab_page));
           d->reload (d->charset);
           break;
          }
      }
*/
}


void document_holder::close_current (void)
{
  int i = tab_widget->currentIndex();
  if (i < 0)
     return;

  CDocument *d = list[i];
  list.removeAt (i);
  delete d;
}


CDocument* document_holder::get_current ()
{
  int i = tab_widget->currentIndex();
  if (i < 0)
     return NULL;

  return list[i];
}


bool CDocument::save_with_name_plain (const QString &fileName)
{
  QFile file (fileName);
  if (! file.open (QFile::WriteOnly | QFile::Text))
      return false;

  QTextStream out (&file);
  out.setCodec (charset.toUtf8());
  out << textEdit->toPlainText();

  return true;
}


QString CDocument::get_filename_at_cursor ()
{
  if (textEdit->textCursor().hasSelection ())
    {
     QFileInfo nf (file_name);
     QDir cd (nf.absolutePath());
     return cd.cleanPath (cd.absoluteFilePath(textEdit->textCursor().selectedText()));
    }

  QString s = textEdit->toPlainText();

  QString x;

  int pos = textEdit->textCursor().position();

  int end = s.indexOf ("\"", pos);
  if (end == -1)
     return x;

  int start = s.lastIndexOf ("\"", pos);
  if (start == -1)
     return x;

  x = s.mid (start + 1, end - (start + 1));

  if (x.startsWith("#"))
     return x;

  QFileInfo inf (file_name);
  QDir cur_dir (inf.absolutePath());

  return cur_dir.cleanPath (cur_dir.absoluteFilePath(x));
}



CSyntaxHighlighter::CSyntaxHighlighter (QTextDocument *parent): QSyntaxHighlighter (parent)
{

}


void CDocument::set_hl()
{
  QFileInfo fi = QFileInfo (file_name);

  if (!fi.exists())
      return;

  QString ext = fi.suffix().toLower();

  if ((ext.endsWith("c")) || (ext.endsWith("cpp")) || (ext.endsWith("h")) || (ext.endsWith("hh")))
         {
          if (highlighter)
             delete highlighter;
          highlighter = new CSyntaxHighlighter(textEdit->document());
          highlighter->load_from_xml (":/hls/clike.xml");
         }
  else
      if ((ext.endsWith("html")) || (ext.endsWith("htm")) || (ext.endsWith("xhtml")) || (ext.endsWith("xml")))
         {
          if (highlighter)
             delete highlighter;
          highlighter = new CSyntaxHighlighter(textEdit->document());
          highlighter->load_from_xml (":/hls/xml.xml");
         }
  else
      if ((ext.endsWith("po")))
         {
          if (highlighter)
             delete highlighter;
          highlighter = new CSyntaxHighlighter(textEdit->document());
          highlighter->load_from_xml (":/hls/po.xml");
         }
}


void CDocument::goto_pos (int pos)
{
  QTextCursor cr = textEdit->textCursor();
  cr.setPosition (pos);
  textEdit->setTextCursor (cr);
}


void document_holder::apply_settings_single (CDocument *d)
{
  QFont f;
  f.fromString (settings->value("editor_font_name", "Monospace").toString());
  f.setPointSize (settings->value("editor_font_size", "12").toInt());
  d->textEdit->setFont(f);

  if (settings->value("word_wrap", "2").toInt() == 2)
     d->textEdit->setLineWrapMode (QTextEdit::WidgetWidth);
  else
      d->textEdit->setLineWrapMode (QTextEdit::NoWrap);

  d->textEdit->setHightlighCurrentLine (settings->value("additional_hl", false).toBool());

  d->cursor_xy_visible = settings->value("cursor_xy_visible", "2").toBool();
}


void document_holder::apply_settings (void)
{
  int c = list.size();

  for (int i = 0; i < c; i++)
      apply_settings_single (list.at(i));
}


image_viewer::image_viewer(const QString &filename)
{
 QLabel *label = new QLabel (this);
 QPixmap pm (filename);
 label->setPixmap (pm);
 resize (pm.size());
 setWindowTitle (strippedName (filename) + " " + QString::number (label->pixmap()->width()) + "x" + QString::number (label->pixmap()->height()));
 show();
}


void document_holder::add_to_recent (CDocument *d)
{
  if (! file_exists (d->file_name))
     return;

  QString s;
  s += d->file_name;
  s += ",";
  s += d->charset;
  s += ",";
  s += QString ("%1").arg (d->textEdit->textCursor().position());

  recent_files.prepend (s);
  if (recent_files.size () > 13)
     recent_files.removeLast ();
}


void document_holder::update_recent_menu ()
{
  recent_menu->clear ();
  create_menu_from_list (this, recent_menu, recent_files, SLOT(open_recent()));
}


void document_holder::open_recent ()
{
  QAction *Act = qobject_cast<QAction *>(sender());

  int i = recent_files.indexOf (Act->text ());
  if (i == -1)
     return;

  open_file_triplex (recent_files[i]);
  //recent_files.removeAt (i);
  update_recent_menu ();
}


CDocument* document_holder::open_file_triplex (const QString &triplex)
{
  QStringList sl = triplex.split (",");
  if (sl.size() != 3)
     return NULL;

  CDocument *d = open_file (sl[0], sl[1]);
  if (d)
     d->goto_pos (sl[2].toInt());

  return d;
}


QString CDocument::get_triplex()
{
  QString s;
  s += file_name;
  s += ",";
  s += charset;
  s += ",";
  s += QString ("%1").arg (textEdit->textCursor().position());
  return s;
}


void CDocument::set_markup_mode ()
{
  markup_mode = holder->markup_mode;

  QString e = file_get_ext (file_name);

  if ( e == "htm" || e == "html")
    markup_mode = "HTML";

  if ( e == "xhtml")
    markup_mode = "XHTML";

  if ( e == "xml")
    markup_mode = "Docbook";
}


CLogMemo::CLogMemo (QTextEdit *m)
{
  memo = m;
  //inf = infm;
}


void CLogMemo::log (const QString &text, const QString &movie)
{
  QTextCursor cr = memo->textCursor();
  cr.movePosition (QTextCursor::Start);
  cr.movePosition (QTextCursor::Down, QTextCursor::MoveAnchor, 0);
  memo->setTextCursor (cr);

  QTime t = QTime::currentTime();

  memo->textCursor().insertHtml ("[" + t.toString("hh:mm:ss") + "] " + text + "<br>");

//  if (! movie.isNull())
  //   inf->change_movie (movie);
}


void CTEAEdit::dragEnterEvent(QDragEnterEvent *event)
{
  if (event->mimeData()->hasFormat("text/uri-list"))
      event->acceptProposedAction();
}


void CTEAEdit::cb_cursorPositionChanged()
{
   if (highlightCurrentLine)
       viewport()->update();

  doc->update_status();
  ensureCursorVisible ();
}


void CTEAEdit::setCurrentLineColor( QColor newColor )
{
        currentLineColor = newColor;
        emit repaint();
}

void CTEAEdit::setHightlighCurrentLine( bool enable )
{
        highlightCurrentLine = enable;
        emit repaint();
}


void CTEAEdit::paintEvent(QPaintEvent *event)
{

  if (highlightCurrentLine)
     {
      QPainter painter (viewport());

     // painter.begin(this);
      QRect r = cursorRect();
      r.setX (0);
      r.setWidth (viewport()->width());
      painter.fillRect (r, QBrush(currentLineColor));
      painter.end();

      QTextCursor cur = textCursor();
      int pos = cur.position ();
      QTextBlock block = document()->findBlock (pos);
      QString tmp = block.text();
      QChar charCC = tmp.at (pos - block.position());

      int cc = tmp.at (pos - block.position ()).toLatin1();
      if (exists (pos))
         switch ( cc )
                {
                 case '{':
                          paintBracketForward (cur, pos, QChar ('{'), QChar ('}'));
                          break;
                 case '}':
                          paintBracketBack(  cur, pos , QChar( '{' ), QChar( '}' ) );
                          break;
                 case '(':
                          paintBracketForward(  cur, pos , QChar( '(' ), QChar( ')' ) );
                          break;
                 case ')':
                          paintBracketBack(  cur, pos , QChar( '(' ), QChar( ')' ) );
                          break;
                 case '[':
                          paintBracketForward(  cur, pos , QChar( '[' ), QChar( ']' ) );
                          break;
                 case ']':
                          paintBracketBack(  cur, pos , QChar( '[' ), QChar( ']' ) );
                          break;
                }
      }

  QTextEdit::paintEvent( event );
}


//CTEAEdit::CTEAEdit (QWidget *parent): QTextEdit(parent), c(0)
CTEAEdit::CTEAEdit (QWidget *parent): QTextEdit (parent)
{
  //completer_enabled = true;
  //completer_enabled = false;

  currentLineColor = QColor( "#EEF6FF" );
  highlightCurrentLine = true;

  connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cb_cursorPositionChanged()));
}


void CTEAEdit::dropEvent(QDropEvent *event)
{
  QList <QUrl> l;
  QString fName;
  QFileInfo info;

  if (event->mimeData()->hasUrls())
     {
      l = event->mimeData()->urls();
      if (l.size() > 0)
         {
          int c = l.size();
          for (int i = 0; i < c; i++)
              {
               fName = l[i].toLocalFile();
               info.setFile(fName);
               if (info.isFile())
                  doc->holder->open_file (fName, "UTF-8");
             }
         }
      }

  event->acceptProposedAction();
}

/*
void CTEAEdit::setCompleter(QCompleter *completer)
{
    if (c)
        QObject::disconnect(c, 0, this, 0);

    c = completer;

    if (! c)
        return;

    c->setWidget (this);
    c->setCompletionMode(QCompleter::PopupCompletion);
    c->setCaseSensitivity(Qt::CaseInsensitive);
    QObject::connect(c, SIGNAL(activated(const QString&)),
                     this, SLOT(insertCompletion(const QString&)));
}


void CTEAEdit::insertCompletion (const QString& completion)
{
    if (c->widget() != this)
        return;

    QTextCursor tc = textCursor();
    int extra = completion.length() - c->completionPrefix().length();
    tc.movePosition(QTextCursor::Left);
    tc.movePosition(QTextCursor::EndOfWord);
    tc.insertText(completion.right(extra));
    setTextCursor(tc);
}


QString CTEAEdit::textUnderCursor() const
{
    QTextCursor tc = textCursor();
    tc.select(QTextCursor::WordUnderCursor);
    return tc.selectedText();
}


void CTEAEdit::focusInEvent(QFocusEvent *e)
{
    if (c)
        c->setWidget(this);
    QTextEdit::focusInEvent(e);
}
*/

//void CTEAEdit::keyPressEvent(QKeyEvent *e)
//{
   //QTextEdit::keyPressEvent(e);
   //doc->update_status ();

/*   if (! completer_enabled)
       {
        QTextEdit::keyPressEvent(e);
        doc->update_status ();
        return;
       }


    if (c && c->popup()->isVisible()) {
        // The following keys are forwarded by the completer to the widget
       switch (e->key()) {
       case Qt::Key_Enter:
       case Qt::Key_Return:
       case Qt::Key_Escape:
       case Qt::Key_Tab:
       case Qt::Key_Backtab:
            e->ignore();
            return; // let the completer do default behavior
       default:
           break;
       }
    }

    bool isShortcut = ((e->modifiers() & Qt::ControlModifier) && e->key() == Qt::Key_E); // CTRL+E
    if (!c || !isShortcut) // dont process the shortcut when we have a completer
        QTextEdit::keyPressEvent(e);

    const bool ctrlOrShift = e->modifiers() & (Qt::ControlModifier | Qt::ShiftModifier);
    if (!c || (ctrlOrShift && e->text().isEmpty()))
        return;

    static QString eow("~!@#$%^&*()_+{}|:\"<>?,./;'[]\\-="); // end of word
    bool hasModifier = (e->modifiers() != Qt::NoModifier) && !ctrlOrShift;
    QString completionPrefix = textUnderCursor();

    if (!isShortcut && (hasModifier || e->text().isEmpty()|| completionPrefix.length() < 3
                      || eow.contains(e->text().right(1)))) {
        c->popup()->hide();
        return;
    }

    if (completionPrefix != c->completionPrefix()) {
        c->setCompletionPrefix(completionPrefix);
        c->popup()->setCurrentIndex(c->completionModel()->index(0, 0));
    }
    QRect cr = cursorRect();
    cr.setWidth(c->popup()->sizeHintForColumn(0)
                + c->popup()->verticalScrollBar()->sizeHint().width());
    c->complete(cr); // popup it up!*/
//}


QStringList CDocument::get_words()
{
  QStringList result;

  QTextCursor cr = textEdit->textCursor();

  cr.setPosition (0);
  cr.movePosition (QTextCursor::Start, QTextCursor::MoveAnchor);

  while (cr.movePosition (QTextCursor::NextWord))
        {
         cr.movePosition (QTextCursor::EndOfWord, QTextCursor::KeepAnchor);
         if (! cr.selectedText().isEmpty())
            result.prepend (cr.selectedText());
        }

  return result;
}



void CSyntaxHighlighter::highlightBlock(const QString &text)
{
   foreach (HighlightingRule rule, highlightingRules)
           {
            QRegExp expression (rule.pattern);
            int index = text.indexOf(expression);
            while (index >= 0)
                  {
                   int length = expression.matchedLength();
                   setFormat(index, length, rule.format);
                   index = text.indexOf(expression, index + length);
                  }
            }

  setCurrentBlockState(0);

  int startIndex = 0;

  if (previousBlockState() != 1)
      startIndex = text.indexOf(commentStartExpression);

  while (startIndex >= 0)
        {
         int endIndex = text.indexOf(commentEndExpression, startIndex);
         int commentLength;
         if (endIndex == -1)
            {
             setCurrentBlockState(1);
             commentLength = text.length() - startIndex;
            }
         else
             commentLength = endIndex - startIndex + commentEndExpression.matchedLength();

         setFormat(startIndex, commentLength, multiLineCommentFormat);
         startIndex = text.indexOf(commentStartExpression, startIndex + commentLength);
        }
}

/*
void CInformer::mousePressEvent(QMouseEvent *event)
{
  if (event->button() == Qt::LeftButton)
     {
      dragPosition = event->globalPos() - frameGeometry().topLeft();
      event->accept();
     }
}


void CInformer::mouseMoveEvent(QMouseEvent *event)
{
  if (event->buttons() & Qt::LeftButton)
     {
      move(event->globalPos() - dragPosition);
      event->accept();
     }
}


void CInformer::movie_finished()
{
  QString fname = "0000.png";
  int i = (qrand() % 5);

  switch (i)
        {
         case 0:
               {
                fname = "0000.png";
                break;
               }

         case 1:
               {
                fname = "0001.png";
                break;
               }

         case 2:
               {
                fname = "0108.png";
                break;
               }

         case 3:
               {
                fname = "0125.png";
                break;
               }

         case 4:
               {
                fname = "0216.png";
                break;
               }
        }

  label->setPixmap (QPixmap (":/mngs/" + fname));
}


void CInformer::change_movie (const QString &file_name)
{
  if (movie->state() == QMovie::Running)
     movie->stop();

  movie->setFileName (":/mngs/"+ file_name);
  label->setMovie (movie);
  movie->start();
}


CInformer::CInformer()
{
  //QRegion r1 (QRect(100, 100, 200, 80), QRegion::Ellipse);
//  QRegion r1 (100, 100, 200, 200, QRegion::Ellipse);
//  setMask (r1);

  setWindowTitle (tr("TEAvisor"));
  QRegion r1 (1, 1, 98, 72);
  resize (98, 72);
  setMask (r1);

  label = new QLabel (this);
  movie = new QMovie (":/mng/wait01.mng");

  connect(movie, SIGNAL(finished ()), this, SLOT(movie_finished ()));

  label->setMovie (movie);
  movie->start();
  show();
}


CInformer::~CInformer()
{
  close();
}
*/

void CSyntaxHighlighter::load_from_xml (const QString &fname)
{

 QDomDocument doc("mydocument");

 if (! doc.setContent (qstring_load (fname)))
    {
     qDebug () << "cannot open";
     return;
    }

 QTextCharFormat tformat;
 QDomElement docElem = doc.documentElement();

 QDomNode n = docElem.firstChild();

 while (! n.isNull())
       {
        QDomElement e = n.toElement();

        if (! e.isNull())
           {
            if (e.attribute("type") == "keywords")
               {
                QString color = e.attribute("color");
                if (! color.isNull() || ! color.isEmpty())
                   tformat.setForeground (QBrush(QColor(color)));
                else
                    tformat.setForeground(Qt::darkBlue);


                QString fontstyle = e.attribute("fontstyle");
                if (! fontstyle.isNull() || !fontstyle.isEmpty())
                   {
                    if (fontstyle.contains ("bold"))
                       tformat.setFontWeight(QFont::Bold);

                    if (fontstyle.contains ("italic"))
                      tformat.setFontItalic(true);
                   }

                QStringList keywordPatterns = e.text().trimmed().split(" ");
                HighlightingRule rule;

                foreach (QString pattern, keywordPatterns)
                        {
                         rule.pattern = QRegExp (pattern);
                         rule.format = tformat;
                         highlightingRules.append(rule);
                        }
                }


        if (e.attribute("type") == "item")
           {
            QString color = e.attribute("color");
            if (! color.isNull() || ! color.isEmpty())
                tformat.setForeground (QBrush(QColor(color)));
             else
                 tformat.setForeground(Qt::darkBlue);


            QString fontstyle = e.attribute("fontstyle");
            if (! fontstyle.isNull() || !fontstyle.isEmpty())
               {
                if (fontstyle.contains ("bold"))
                    tformat.setFontWeight(QFont::Bold);

                if (fontstyle.contains ("italic"))
                    tformat.setFontItalic(true);
               }

            HighlightingRule rule;
            rule.pattern = QRegExp (e.text().trimmed());
            rule.format = tformat;
            highlightingRules.append(rule);
           }

        if (e.attribute("type") == "mcomment-start")
           {
            QString color = e.attribute("color");
            if (! color.isNull() || ! color.isEmpty())
               tformat.setForeground (QBrush(QColor(color)));
            else
                tformat.setForeground(Qt::darkBlue);


            QString fontstyle = e.attribute("fontstyle");
            if (! fontstyle.isNull() || !fontstyle.isEmpty())
               {
                if (fontstyle.contains ("bold"))
                    tformat.setFontWeight(QFont::Bold);

                if (fontstyle.contains ("italic"))
                    tformat.setFontItalic(true);
               }

             multiLineCommentFormat = tformat;
             commentStartExpression = QRegExp(e.text().trimmed());
           }

        if (e.attribute("type") == "mcomment-end")
             commentEndExpression = QRegExp(e.text().trimmed());
       }

     n = n.nextSibling();
    }
}


//
void CTEAEdit::paintBracketForward(  QTextCursor & cur, int pos , const QChar &begin, const QChar &end)
{
        QBrush brush( Qt::yellow );
        QPainter painter( viewport() );
        QTextBlock block = document()->findBlock( pos );
        cur.movePosition( QTextCursor::NextCharacter );
        QRect r = cursorRect();
        QFontMetrics * fm = new QFontMetrics( block.charFormat ().font () );
        r.setX( r.x() + fm->width("a") );
        r.setWidth( fm->width( begin ) ); painter.fillRect( r, brush );
        int ends = ending( pos, begin );
        if ( ends != -1 )
        {
                cur.movePosition( QTextCursor::NextCharacter, QTextCursor::MoveAnchor, ends - pos );
                r = cursorRect(cur);
                r.setWidth( fm->width( begin ) );
                painter.fillRect( r, brush );
        }
}

void CTEAEdit::paintBracketBack(  QTextCursor & cur, int pos , const QChar & begin, const QChar & end )
{
        QBrush brush( Qt::yellow );
        QPainter painter( viewport() );
        QTextBlock block = document()->findBlock( pos );
        cur.movePosition( QTextCursor::NextCharacter );
        QRect r = cursorRect();
        QFontMetrics * fm = new QFontMetrics( block.charFormat ().font () );
        r.setX( r.x() + fm->width("a") );
        r.setWidth( fm->width( end ) ); painter.fillRect( r, brush );
        int forwards = forwarding( pos, begin );
        if ( forwards != -1 )
        {
                cur.movePosition( QTextCursor::PreviousCharacter, QTextCursor::MoveAnchor, pos - forwards );
                r = cursorRect(cur);
                r.setWidth( fm->width( begin ) );
                painter.fillRect( r, brush );
        }
}

bool CTEAEdit::exists( int pos )
{
        TextType = Text_OFF;
        bool ret = false;
        int cc;
        QString tmp = document()->toPlainText ();
        for( int i = 0; i < pos; i++ )
        {
                cc = tmp.at( i ).toLatin1 ();
                switch ( TextType )
                {
                case Text_OFF:
                        if ( cc == '\'' ) TextType = Text_CHAR;
                        if ( cc == '"' ) TextType = Text_STRING;
                        if (i > 0 && cc == '*' && tmp.at( i - 1 ).toLatin1 () == '/') TextType = Text_MULTILINECOMMENT;
                        if (i > 0 && cc == '/' && tmp.at( i - 1 ).toLatin1 () == '/') TextType = Text_SINGLELINECOMMENT;
                        break;
                case Text_SINGLELINECOMMENT: if ( cc == 0x0A ) TextType =  Text_OFF; break;
                case Text_MULTILINECOMMENT: if (i > 0 && cc == '/' && tmp.at( i - 1 ).toLatin1 () == '*') TextType = Text_OFF; break;
                case Text_CHAR: if ( cc == '\'' ) TextType = Text_OFF; break;
                case Text_STRING: if ( cc == '"' ) TextType = Text_OFF; break;
                }
        }
        if ( TextType == Text_OFF ) ret = true;
        return ret;
}


int CTEAEdit::ending( int pos, const QChar & ch )
{
        int count = 0;
        int current = -1;
        QChar begin = ch, end;
        switch ( begin.toLatin1() )
        {
        case '{': end = QChar( '}' ); break;
        case '(': end = QChar( ')' ); break;
        case '[': end = QChar( ']' ); break;
        }
        TextType = Text_OFF;
        int cc;
        QString tmp = document()->toPlainText ();
        for( int i = pos; i < tmp.length(); i++ )
        {
                cc = tmp.at( i ).toLatin1 ();
                switch ( TextType )
                {
                case Text_OFF:
                        if ( cc == '\'' ) TextType = Text_CHAR;
                        if ( cc == '"' ) TextType = Text_STRING;
                        if (i > 0 && cc == '*' && tmp.at( i - 1 ).toLatin1 () == '/') TextType = Text_MULTILINECOMMENT;
                        if (i > 0 && cc == '/' && tmp.at( i - 1 ).toLatin1 () == '/') TextType = Text_SINGLELINECOMMENT;
                        if ( cc == end.toLatin1 () ) { count--; if ( count == 0 ) { current = i; break;} }
                        if ( cc == begin.toLatin1 () )  count++; break;
                case Text_SINGLELINECOMMENT: if ( cc == 0x0A ) TextType =  Text_OFF; break;
                case Text_MULTILINECOMMENT: if (i > 0 && cc == '/' && tmp.at( i - 1 ).toLatin1 () == '*') TextType = Text_OFF; break;
                case Text_CHAR: if ( cc == '\'' ) TextType = Text_OFF; break;
                case Text_STRING: if ( cc == '"' ) TextType = Text_OFF; break;
                }
                if ( current != -1 ) break;
        }
        return current;
}


int CTEAEdit::forwarding( int pos, const QChar & ch )
{
        QList<QChar>  lstCh;
        QList<int> lstPos;
        int count = 0, i;
        int current = -1;
        QChar begin = ch, end;
        switch ( begin.toLatin1() )
        {
        case '{': end = QChar( '}' ); break;
        case '(': end = QChar( ')' ); break;
        case '[': end = QChar( ']' ); break;
        }
        TextType = Text_OFF;
        int cc;
        QString tmp = document()->toPlainText ();
        for( i = 0; i < pos + 1; i++ )
        {
                cc = tmp.at( i ).toLatin1 ();
                switch ( TextType )
                {
                case Text_OFF:
                        if ( cc == '\'' ) TextType = Text_CHAR;
                        if ( cc == '"' ) TextType = Text_STRING;
                        if (i > 0 && cc == '*' && tmp.at( i - 1 ).toLatin1 () == '/') TextType = Text_MULTILINECOMMENT;
                        if (i > 0 && cc == '/' && tmp.at( i - 1 ).toLatin1 () == '/') TextType = Text_SINGLELINECOMMENT;
                        if ( cc == end.toLatin1 () ) {  lstCh.append( end );  lstPos.append( i );   }
                        if ( cc == begin.toLatin1 () ) { lstCh.append( begin );  lstPos.append( i ); }
                        break;
                case Text_SINGLELINECOMMENT: if ( cc == 0x0A ) TextType =  Text_OFF; break;
                case Text_MULTILINECOMMENT: if (i > 0 && cc == '/' && tmp.at( i - 1 ).toLatin1 () == '*') TextType = Text_OFF; break;
                case Text_CHAR: if ( cc == '\'' ) TextType = Text_OFF; break;
                case Text_STRING: if ( cc == '"' ) TextType = Text_OFF; break;
                }
        }
        count = 0;
        int index = lstPos.indexOf( pos );
        for ( i = index; i >=0; i-- )
        {
                if ( lstCh.at( i ) == end ) count++; else count--;
                if ( count == 0 ) { current = lstPos.at( i ); break;  }
        }

        return current;
}


CTEAEdit::~CTEAEdit()
{
qDebug() << "~CTEAEdit";
//setAttribute(Qt::WA_DeleteOnClose);
}


void document_holder::save_to_session (const QString &fileName)
{
  CDocument *d;
  int c = list.size();
  if (c < 0)
     return;

  fname_current_session = fileName;
  QString l;

  for (int i = 0; i < c; i++)
      {
       d = list[i];
       l += d->get_triplex();
       l += "\n";
      }

  qstring_save (fileName, l.trimmed());
}


void document_holder::load_from_session (const QString &fileName)
{
  if (! file_exists(fileName))
     return;

  QStringList l = qstring_load (fileName).split("\n");
  int c = l.size();
  if (c < 0)
     return;

  fname_current_session = fileName;

  for (int i = 0; i < c; i++)
      open_file_triplex (l[i]);
}


